﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace NetworkCommon
{
  /// <summary>
  /// Message that informs a client that their connection is being closed
  /// </summary>
  public class MsgCloseConnection : Message
  {
    /// <summary>
    /// Creates a close connection message
    /// </summary>
    public MsgCloseConnection()
    {
    }
    
    /// <summary>
    /// Creates a close connection message from the specified byte array
    /// that contains the data portion of a received network message
    /// </summary>
    /// <param name="data">
    /// data portion of the message
    /// </param>
    public MsgCloseConnection(byte[] data)
    {
      // this message does not contain any data, so the data portion is
      // irrelevant
    }
    
    /// <summary>
    /// Retrieves the message id
    /// </summary>
    /// <returns>
    /// message id
    /// </returns>
    override public MessageIDs GetMessageID()
    {
      return MessageIDs.CLOSE_CONNECTION;
    }
    
    /// <summary>
    /// Puts the any needed data into the message via the specified binary
    /// writer
    /// </summary>
    /// <param name="writer">
    /// where to write the data to
    /// </param>
    /// <returns>
    /// number of bytes written to writer
    /// </returns>
    override protected int WriteContents(BinaryWriter writer)
    {
      // no need to do anything for this message
      return 0;
    }
  }
}
