﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework;

namespace Particle_System_Editor
{
  public partial class FormEditColorSeq : Form
  {
    /// <summary>
    /// Creates a new dialog that allows a user to add/edit a color entry for a
    /// Color Sequence
    /// </summary>
    /// <param name="color">
    /// Color to use for the entry before the user changes it
    /// </param>
    /// <param name="life">
    /// Percent of a particle's life, in the range [0,100], that the color
    /// should be used to color a particle before the user changes it
    /// </param>
    public FormEditColorSeq(Vector3 color,decimal life)
    {
      InitializeComponent();
      
      AcceptButton = ok;
      CancelButton = cancel;
      
      color_display.BackColor = color.ToSystemColor();
      percent_life.Value = life;
    }
    
    /// <summary>
    /// Retrieves the color the user selected.  Should only be called if the
    /// dialog result is OK
    /// </summary>
    /// <returns>
    /// Selected color
    /// </returns>
    public Vector3 GetColor()
    {
      return color_display.BackColor.ToRenderingColor();
    }
    
    /// <summary>
    /// Retrieves the percent of a particle's life, in the range [0,100], that
    /// the color should be used to color a particle.  Should only be called
    /// if the dialog result is OK
    /// </summary>
    /// <returns>
    /// </returns>
    public decimal GetLife()
    {
      return percent_life.Value;
    }

    /// <summary>
    /// Handles the ok button being clicked
    /// </summary>
    private void ok_Click(object sender,EventArgs e)
    {
      DialogResult = System.Windows.Forms.DialogResult.OK;
    }
    
    /// <summary>
    /// Handles the cancel button being clicked
    /// </summary>
    private void cancel_Click(object sender,EventArgs e)
    {
      DialogResult = System.Windows.Forms.DialogResult.Cancel;
    }
    
    /// <summary>
    /// Handles the change color button being clicked by allowing the user to
    /// change the color
    /// </summary>
    private void change_color_Click(object sender,EventArgs e)
    {
      color_select.Color = color_display.BackColor;
      if (color_select.ShowDialog() == System.Windows.Forms.DialogResult.OK)
      {
        color_display.BackColor = color_select.Color;
      }
    }
  }
}
