﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.Windows.Forms;
using System.Xml;
using Microsoft.Xna.Framework;
using NetworkCommon;

namespace Particle_System_Editor
{
  public partial class FormMain : Form
  {
    /// <summary>
    /// Creates a new main form for the application
    /// </summary>
    public FormMain()
    {
      InitializeComponent();
      
      // setup the map of single value UI components to ID
      m_ids = new Dictionary<object,ParameterIDs>();
      m_ids.Add(cam_lookat_x,      ParameterIDs.CAM_LOOKAT_X);
      m_ids.Add(cam_lookat_y,      ParameterIDs.CAM_LOOKAT_Y);
      m_ids.Add(cam_lookat_z,      ParameterIDs.CAM_LOOKAT_Z);
      m_ids.Add(cam_orbit_range,   ParameterIDs.CAM_ORBIT_RANGE);
      m_ids.Add(cam_orbit_yaw,     ParameterIDs.CAM_ORBIT_YAW);
      m_ids.Add(cam_orbit_pitch,   ParameterIDs.CAM_ORBIT_PITCH);
      m_ids.Add(emitter_shape,     ParameterIDs.EMITTER_SHAPE);
      m_ids.Add(emitter_pos_x,     ParameterIDs.EMITTER_POS_X);
      m_ids.Add(emitter_pos_y,     ParameterIDs.EMITTER_POS_Y);
      m_ids.Add(emitter_pos_z,     ParameterIDs.EMITTER_POS_Z);
      m_ids.Add(emitter_rot_yaw,   ParameterIDs.EMITTER_ROT_YAW);
      m_ids.Add(emitter_rot_pitch, ParameterIDs.EMITTER_ROT_PITCH);
      m_ids.Add(emitter_rot_roll,  ParameterIDs.EMITTER_ROT_ROLL);
      m_ids.Add(emitter_scale_x,   ParameterIDs.EMITTER_SCALE_X);
      m_ids.Add(emitter_scale_y,   ParameterIDs.EMITTER_SCALE_Y);
      m_ids.Add(emitter_scale_z,   ParameterIDs.EMITTER_SCALE_Z);
      m_ids.Add(emitter_spawn_time,ParameterIDs.EMITTER_SPAWN_TIME);
      m_ids.Add(emitter_spawn_rate,ParameterIDs.EMITTER_SPAWN_RATE);
      m_ids.Add(pc_dis_type,       ParameterIDs.PARTCILE_DISPERSION_TYPE);
      m_ids.Add(pc_dis_dir_x,      ParameterIDs.PARTICLE_DISPERSION_DIR_X);
      m_ids.Add(pc_dis_dir_y,      ParameterIDs.PARTICLE_DISPERSION_DIR_Y);
      m_ids.Add(pc_dis_dir_z,      ParameterIDs.PARTICLE_DISPERSION_DIR_Z);
      m_ids.Add(pc_dis_angle_min_x,
        ParameterIDs.PARTICLE_DISPERSION_PAR_SPHERE_MIN_X);
      m_ids.Add(pc_dis_angle_min_y,
        ParameterIDs.PARTICLE_DISPERSION_PAR_SPHERE_MIN_Y);
      m_ids.Add(pc_dis_angle_max_x,
        ParameterIDs.PARTICLE_DISPERSION_PAR_SPHERE_MAX_X);
      m_ids.Add(pc_dis_angle_max_y,
        ParameterIDs.PARTICLE_DISPERSION_PAR_SPHERE_MAX_Y);
      m_ids.Add(pc_shape,          ParameterIDs.PARTICLE_SHAPE);
      m_ids.Add(pc_lifetime,       ParameterIDs.PARTICLE_LIFETIME);
      m_ids.Add(pc_min_speed,      ParameterIDs.PARTICLE_MIN_SPEED);
      m_ids.Add(pc_max_speed,      ParameterIDs.PARTICLE_MAX_SPEED);
      m_ids.Add(pc_size,           ParameterIDs.PARTICLE_SIZE);
      m_ids.Add(pc_acc_x,          ParameterIDs.PARTICLE_ACC_X);
      m_ids.Add(pc_acc_y,          ParameterIDs.PARTICLE_ACC_Y);
      m_ids.Add(pc_acc_z,          ParameterIDs.PARTICLE_ACC_Z);
      m_ids.Add(coloring_mode,     ParameterIDs.COLORING_MODE);
      m_ids.Add(coloring_texture,  ParameterIDs.COLORING_TEXTURE_PATH);
      m_ids.Add(coloring_color_seq_interpolate,
        ParameterIDs.COLORING_SEQ_INTERPOLATE);
      
      // update all single value UI components to call the value changed
      // handler
      cam_lookat_x.ValueChanged       += new System.EventHandler(ValueChanged);
      cam_lookat_y.ValueChanged       += new System.EventHandler(ValueChanged);
      cam_lookat_z.ValueChanged       += new System.EventHandler(ValueChanged);
      cam_orbit_range.ValueChanged    += new System.EventHandler(ValueChanged);
      cam_orbit_yaw.ValueChanged      += new System.EventHandler(ValueChanged);
      cam_orbit_pitch.ValueChanged    += new System.EventHandler(ValueChanged);
      emitter_shape.SelectedValueChanged +=
        new System.EventHandler(ValueChanged);
      emitter_pos_x.ValueChanged      += new System.EventHandler(ValueChanged);
      emitter_pos_y.ValueChanged      += new System.EventHandler(ValueChanged);
      emitter_pos_z.ValueChanged      += new System.EventHandler(ValueChanged);
      emitter_rot_yaw.ValueChanged    += new System.EventHandler(ValueChanged);
      emitter_rot_pitch.ValueChanged  += new System.EventHandler(ValueChanged);
      emitter_rot_roll.ValueChanged   += new System.EventHandler(ValueChanged);
      emitter_scale_x.ValueChanged    += new System.EventHandler(ValueChanged);
      emitter_scale_y.ValueChanged    += new System.EventHandler(ValueChanged);
      emitter_scale_z.ValueChanged    += new System.EventHandler(ValueChanged);
      emitter_spawn_time.ValueChanged += new System.EventHandler(ValueChanged);
      emitter_spawn_rate.ValueChanged += new System.EventHandler(ValueChanged);
      pc_dis_type.SelectedValueChanged+= new System.EventHandler(ValueChanged);
      pc_dis_dir_x.ValueChanged       += new System.EventHandler(ValueChanged);
      pc_dis_dir_y.ValueChanged       += new System.EventHandler(ValueChanged);
      pc_dis_dir_z.ValueChanged       += new System.EventHandler(ValueChanged);
      pc_dis_angle_min_x.ValueChanged += new System.EventHandler(ValueChanged);
      pc_dis_angle_max_x.ValueChanged += new System.EventHandler(ValueChanged);
      pc_dis_angle_min_y.ValueChanged += new System.EventHandler(ValueChanged);
      pc_dis_angle_max_y.ValueChanged += new System.EventHandler(ValueChanged);
      pc_shape.SelectedValueChanged   += new System.EventHandler(ValueChanged);
      pc_lifetime.ValueChanged        += new System.EventHandler(ValueChanged);
      pc_min_speed.ValueChanged       += new System.EventHandler(ValueChanged);
      pc_max_speed.ValueChanged       += new System.EventHandler(ValueChanged);
      pc_size.ValueChanged            += new System.EventHandler(ValueChanged);
      pc_acc_x.ValueChanged           += new System.EventHandler(ValueChanged);
      pc_acc_y.ValueChanged           += new System.EventHandler(ValueChanged);
      pc_acc_z.ValueChanged           += new System.EventHandler(ValueChanged);
      coloring_mode.SelectedValueChanged
        += new System.EventHandler(ValueChanged);
      coloring_texture.TextChanged    += new System.EventHandler(ValueChanged);
      coloring_color_seq_interpolate.CheckedChanged
        += new System.EventHandler(ValueChanged);
      
      // setup the owner-draw functions for drawing the color set/sequence list
      // boxes
      coloring_color_set.DrawItem += new DrawItemEventHandler(
        coloring_color_set_DrawItem);
      coloring_color_seq.DrawItem += new DrawItemEventHandler(
        coloring_color_seq_DrawItem);
      
      // make sure combo boxes have their first element selected
      emitter_shape.SelectedIndex = 0;
      pc_shape.SelectedIndex      = 0;
      pc_dis_type.SelectedIndex   = 0;
      coloring_mode.SelectedIndex = 0;
      
      // update menu items for connecting/disconnecting from the server
      connectToolStripMenuItem.Enabled = true;
      disconnectToolStripMenuItem.Enabled = false;
      
      // setup the delegate for processing messages from the server
      receive_msg = ReceiveMsg;
      
      // init the save file name
      m_filename = "";
    }

    /// <summary>
    /// Event handler for UI elements with a single value when their value
    /// changes
    /// </summary>
    private void ValueChanged(object sender,EventArgs e)
    {
      ParameterIDs id;
      if (m_ids.TryGetValue(sender,out id))
      {
        lock (this)
        {
          if (m_conn != null)
          {
            // create the message (different types of messages needed depending
            // on what was set)
            NetworkCommon.Message msg = null;
            if (sender is TextBox)
            {
              TextBox input = (TextBox)sender;
              msg = new NetworkCommon.MsgSetString(id,input.Text);
            }
            else if (sender is NumericUpDown)
            {
              NumericUpDown input = (NumericUpDown)sender;
              msg = new NetworkCommon.MsgSetSingle(id,(float)input.Value);
            }
            else if (sender is ComboBox)
            {
              ComboBox input = (ComboBox)sender;
              msg = new NetworkCommon.MsgSetSingle(id,
                (float)input.SelectedIndex);
            }
            else if (sender is CheckBox)
            {
              CheckBox input = (CheckBox)sender;
              msg = new NetworkCommon.MsgSetSingle(id,
                input.Checked ? 1 : 0);
            }
            // ignore other types of UI elements
            
            // send the message
            if (msg != null)
            {
              msg.Send(m_conn.GetStream());
            }
          }
        }
      }
      else
      {
        // report the no match (mainly a debugging tool when adding new UI
        // elements so it is not forgotten)
        MessageBox.Show("Unknown UI element \"" + sender.ToString() + "\"");
      }
    }

    /// <summary>
    /// Handles the distribution type changing by enabling/disabling UI
    /// elements appropriately
    /// </summary>
    private void pc_dis_type_SelectedIndexChanged(object sender,EventArgs e)
    {
      // disable/enable other particle dispersion UI elements based on the
      // currently selected dispersion type
      if (pc_dis_type.SelectedIndex == 0)
      {
        // directional
        
        // enable needed UI elements
        pc_dis_dir.Enabled       = true;
        pc_dis_dir_x.Enabled     = true;
        pc_dis_dir_y.Enabled     = true;
        pc_dis_dir_z.Enabled     = true;
        lbl_pc_dis_dir_x.Enabled = true;
        lbl_pc_dis_dir_y.Enabled = true;
        lbl_pc_dis_dir_z.Enabled = true;
        
        // disable irrelevant UI elements
        pc_dis_angles.Enabled          = false;
        pc_dis_angle_min_x.Enabled     = false;
        pc_dis_angle_max_x.Enabled     = false;
        pc_dis_angle_min_y.Enabled     = false;
        pc_dis_angle_max_y.Enabled     = false;
        lbl_pc_dis_angle_min_x.Enabled = false;
        lbl_pc_dis_angle_max_x.Enabled = false;
        lbl_pc_dis_angle_min_y.Enabled = false;
        lbl_pc_dis_angle_max_y.Enabled = false;
      }
      else if (pc_dis_type.SelectedIndex == 1)
      {
        // spherical
        
        // no UI elements to enable for this type
        
        // disable irrelevant UI elements
        pc_dis_dir.Enabled             = false;
        pc_dis_dir_x.Enabled           = false;
        pc_dis_dir_y.Enabled           = false;
        pc_dis_dir_z.Enabled           = false;
        lbl_pc_dis_dir_x.Enabled       = false;
        lbl_pc_dis_dir_y.Enabled       = false;
        lbl_pc_dis_dir_z.Enabled       = false;
        pc_dis_angles.Enabled          = false;
        pc_dis_angle_min_x.Enabled     = false;
        pc_dis_angle_max_x.Enabled     = false;
        pc_dis_angle_min_y.Enabled     = false;
        pc_dis_angle_max_y.Enabled     = false;
        lbl_pc_dis_angle_min_x.Enabled = false;
        lbl_pc_dis_angle_max_x.Enabled = false;
        lbl_pc_dis_angle_min_y.Enabled = false;
        lbl_pc_dis_angle_max_y.Enabled = false;
      }
      else /* if (pc_dis_type.SelectedIndex == 2) */
      {
        // partial sphere
        
        // enable needed UI elements
        pc_dis_angles.Enabled          = true;
        pc_dis_angle_min_x.Enabled     = true;
        pc_dis_angle_max_x.Enabled     = true;
        pc_dis_angle_min_y.Enabled     = true;
        pc_dis_angle_max_y.Enabled     = true;
        lbl_pc_dis_angle_min_x.Enabled = true;
        lbl_pc_dis_angle_max_x.Enabled = true;
        lbl_pc_dis_angle_min_y.Enabled = true;
        lbl_pc_dis_angle_max_y.Enabled = true;
        
        // disable irrelevant UI elements
        pc_dis_dir.Enabled       = false;
        pc_dis_dir_x.Enabled     = false;
        pc_dis_dir_y.Enabled     = false;
        pc_dis_dir_z.Enabled     = false;
        lbl_pc_dis_dir_x.Enabled = false;
        lbl_pc_dis_dir_y.Enabled = false;
        lbl_pc_dis_dir_z.Enabled = false;
      }
    }

    /// <summary>
    /// Handles the coloring mode changing by enabling/disabling UI elements
    /// appropriately
    /// </summary>
    private void coloring_mode_SelectedIndexChanged(object sender,EventArgs e)
    {
      // disable/enable other particle coloring UI elements based on the
      // currently selected coloring type
      if (coloring_mode.SelectedIndex == 0)
      {
        // texture
        
        // enable needed UI elements
        coloring_texture.Enabled     = true;
        coloring_browse.Enabled      = true;
        lbl_coloring_texture.Enabled = true;
        
        // disable irrelevant UI elements
        coloring_color_set_group.Enabled  = false;
        coloring_color_set.Enabled        = false;
        coloring_color_set_add.Enabled    = false;
        coloring_color_set_edit.Enabled   = false;
        coloring_color_set_remove.Enabled = false;
        coloring_color_seq_group.Enabled  = false;
        coloring_color_seq.Enabled        = false;
        coloring_color_seq_add.Enabled    = false;
        coloring_color_seq_edit.Enabled   = false;
        coloring_color_seq_remove.Enabled = false;
        coloring_color_seq_up.Enabled     = false;
        coloring_color_seq_down.Enabled   = false;
      }
      else if (coloring_mode.SelectedIndex == 1)
      {
        // color set
        
        // enable needed UI elements
        bool enable = coloring_color_set.SelectedIndex != -1;
        coloring_color_set_group.Enabled  = true;
        coloring_color_set.Enabled        = true;
        coloring_color_set_add.Enabled    = true;
        coloring_color_set_edit.Enabled   = enable;
        coloring_color_set_remove.Enabled = enable;
        
        // disable irrelevant UI elements
        coloring_texture.Enabled          = false;
        coloring_browse.Enabled           = false;
        lbl_coloring_texture.Enabled      = false;
        coloring_color_seq_group.Enabled  = false;
        coloring_color_seq.Enabled        = false;
        coloring_color_seq_add.Enabled    = false;
        coloring_color_seq_edit.Enabled   = false;
        coloring_color_seq_remove.Enabled = false;
        coloring_color_seq_up.Enabled     = false;
        coloring_color_seq_down.Enabled   = false;
      }
      else /* if (coloring_mode.SelectedIndex == 2) */
      {
        // color sequence
        
        // enable needed UI elements
        bool enable = coloring_color_seq.SelectedIndex != -1;
        coloring_color_seq_group.Enabled  = true;
        coloring_color_seq.Enabled        = true;
        coloring_color_seq_add.Enabled    = true;
        coloring_color_seq_edit.Enabled   = enable;
        coloring_color_seq_remove.Enabled = enable;
        coloring_color_seq_up.Enabled     =
          coloring_color_seq.SelectedIndex > 0;
        coloring_color_seq_down.Enabled   = 
          coloring_color_seq.SelectedIndex
          < (coloring_color_seq.Items.Count - 1);
        
        // disable irrelevant UI elements
        coloring_texture.Enabled          = false;
        coloring_browse.Enabled           = false;
        lbl_coloring_texture.Enabled      = false;
        coloring_color_set_group.Enabled  = false;
        coloring_color_set.Enabled        = false;
        coloring_color_set_add.Enabled    = false;
        coloring_color_set_edit.Enabled   = false;
        coloring_color_set_remove.Enabled = false;
      }
    }
    
    /// <summary>
    /// Takes care of drawing each element in the color set list box
    /// </summary>
    private void coloring_color_set_DrawItem(object sender,
      DrawItemEventArgs e)
    {
      // get the color set entry for this item
      ColorSetEntry entry = (ColorSetEntry)coloring_color_set.Items[e.Index];
      
      // take care of the background color
      DrawItemEventArgs draw = new DrawItemEventArgs(e.Graphics,e.Font,
        e.Bounds,e.Index,e.State,e.ForeColor,entry.Color.ToSystemColor());
      draw.DrawBackground();
      
      // display text for the probability
      System.Drawing.Brush brush =
        new System.Drawing.SolidBrush(draw.ForeColor);
      draw.Graphics.DrawString(entry.Chance.ToString(),draw.Font,
        brush,draw.Bounds);
      
      // take care of the focus rect
      e.DrawFocusRectangle();
    }
    
    /// <summary>
    /// Takes care of drawing each element in the color sequence list box
    /// </summary>
    private void coloring_color_seq_DrawItem(object sender,
      DrawItemEventArgs e)
    {
      // get the color set entry for this item
      ColorSeqEntry entry = (ColorSeqEntry)coloring_color_seq.Items[e.Index];
      
      // take care of the background color
      DrawItemEventArgs draw = new DrawItemEventArgs(e.Graphics,e.Font,
        e.Bounds,e.Index,e.State,e.ForeColor,entry.Color.ToSystemColor());
      draw.DrawBackground();
      
      // display text for the probability
      System.Drawing.Brush brush =
        new System.Drawing.SolidBrush(draw.ForeColor);
      draw.Graphics.DrawString(entry.Life.ToString(),draw.Font,
        brush,draw.Bounds);
      
      // take care of the focus rect
      e.DrawFocusRectangle();
    }

    /// <summary>
    /// Enables/disables the UI elements that allow for editing and removing
    /// items from the color set based on whether an item in the color set list
    /// box is selected or not.
    /// </summary>
    private void coloring_color_set_SelectedIndexChanged(object sender,
      EventArgs e)
    {
      bool enable = coloring_color_set.SelectedIndex != -1;
      
      coloring_color_set_edit.Enabled   = enable;
      coloring_color_set_remove.Enabled = enable;
    }
    
    /// <summary>
    /// Enables/disables the UI elements that allow for editing and removing
    /// items from the color sequence based on whether an item in the color
    /// sequence list box is selected or not.
    /// </summary>
    private void coloring_color_seq_SelectedIndexChanged(object sender,
      EventArgs e)
    {
      bool enable = coloring_color_seq.SelectedIndex != -1;
      
      coloring_color_seq_edit.Enabled   = enable;
      coloring_color_seq_remove.Enabled = enable;
      coloring_color_seq_up.Enabled     = 
          coloring_color_seq.SelectedIndex > 0;
      coloring_color_seq_down.Enabled   = 
          coloring_color_seq.SelectedIndex
          < (coloring_color_seq.Items.Count - 1);
    }

    /// <summary>
    /// Handles the texture selection browse button being clicked by allowing
    /// the user to select a file from their file system
    /// </summary>
    private void coloring_browse_Click(object sender,EventArgs e)
    {
      // display a file selection dialog and put the result in coloring_texture
      OpenFileDialog dlg = new OpenFileDialog();
      if (coloring_texture.Text != "")
      {
        dlg.FileName = coloring_texture.Text;
      }
      if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
      {
        coloring_texture.Text = dlg.FileName;
      }
    }
    
    /// <summary>
    /// Handles the color set add button being clicked by displaying the UI for
    /// adding/editing a color set entry
    /// </summary>
    private void coloring_color_set_add_Click(object sender,EventArgs e)
    {
      FormEditColorSet dialog = new FormEditColorSet(Vector3.One,100);
      if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
      {
        // update the UI
        coloring_color_set.BeginUpdate();
        coloring_color_set.Items.Add(
          new ColorSetEntry(dialog.GetColor(),(float)dialog.GetChance()));
        coloring_color_set.EndUpdate();
        
        // inform the server
        SendColorSet();
      }
    }
    
    /// <summary>
    /// Handles the color set edit button being clicked by displaying the UI for
    /// adding/editing a color set entry
    /// </summary>
    private void coloring_color_set_edit_Click(object sender,EventArgs e)
    {
      ColorSetEntry entry = (ColorSetEntry)coloring_color_set.SelectedItem;
      FormEditColorSet dialog =
        new FormEditColorSet(entry.Color,(decimal)entry.Chance);
      if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
      {
        // update the UI
        entry.Color  = dialog.GetColor();
        entry.Chance = (float)dialog.GetChance();
        
        // force a redraw
        coloring_color_set.Invalidate();
        
        // inform the server
        SendColorSet();
      }
    }
    
    /// <summary>
    /// Handles the color set remove button being clicked by removing the
    /// currently selected color set entry from the color set
    /// </summary>
    private void coloring_color_set_remove_Click(object sender,EventArgs e)
    {
      // update the UI
      coloring_color_set.Items.RemoveAt(coloring_color_set.SelectedIndex);
      
      // inform the server
      SendColorSet();
    }

    /// <summary>
    /// Handles the color sequence add button being clicked by displaying the
    /// UI for adding/editing a color sequence entry
    /// </summary>
    private void coloring_color_seq_add_Click(object sender,EventArgs e)
    {
      FormEditColorSeq dialog = new FormEditColorSeq(Vector3.One,100);
      if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
      {
        // update the UI
        coloring_color_seq.BeginUpdate();
        coloring_color_seq.Items.Add(
          new ColorSeqEntry(dialog.GetColor(),(float)dialog.GetLife()));
        coloring_color_seq.EndUpdate();
        
        // inform the server
        SendColorSeq();
      }
    }
    
    /// <summary>
    /// Handles the color sequence edit button being clicked by displaying the
    /// UI for adding/editing a color sequence entry
    /// </summary>
    private void coloring_color_seq_edit_Click(object sender,EventArgs e)
    {
      ColorSeqEntry entry = (ColorSeqEntry)coloring_color_seq.SelectedItem;
      FormEditColorSeq dialog =
        new FormEditColorSeq(entry.Color,(decimal)entry.Life);
      if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
      {
        // update the UI
        entry.Color = dialog.GetColor();
        entry.Life  = (float)dialog.GetLife();
        
        // force a redraw
        coloring_color_seq.Invalidate();
        
        // inform the server
        SendColorSeq();
      }
    }
    
    /// <summary>
    /// Handles the color sequence remove button being clicked by removing the
    /// currently selected color sequence entry from the color sequence
    /// </summary>
    private void coloring_color_seq_remove_Click(object sender,EventArgs e)
    {
      // update the UI
      coloring_color_seq.Items.RemoveAt(coloring_color_seq.SelectedIndex);
      
      // inform the server
      SendColorSeq();
    }

    /// <summary>
    /// Handles the color sequence up button being clicked by moving the
    /// currently selected entry up one position
    /// </summary>
    private void coloring_color_seq_up_Click(object sender,EventArgs e)
    {
      int i = coloring_color_seq.SelectedIndex;
      Object tmp = coloring_color_seq.Items[i];
      coloring_color_seq.Items[i] = coloring_color_seq.Items[i - 1];
      coloring_color_seq.Items[i - 1] = tmp;
      
      --coloring_color_seq.SelectedIndex;
      
      coloring_color_seq.Invalidate();
      
      // inform the server
      SendColorSeq();
    }
    
    /// <summary>
    /// Handles the color sequence down button being clicked by moving the
    /// currently selected entry down one position
    /// </summary>
    private void coloring_color_seq_down_Click(object sender,EventArgs e)
    {
      int i = coloring_color_seq.SelectedIndex;
      Object tmp = coloring_color_seq.Items[i];
      coloring_color_seq.Items[i] = coloring_color_seq.Items[i + 1];
      coloring_color_seq.Items[i + 1] = tmp;
      
      ++coloring_color_seq.SelectedIndex;
      
      coloring_color_seq.Invalidate();
      
      // inform the server
      SendColorSeq();
    }

    /// <summary>
    /// Sends the current color set to the rendering server.  This should be
    /// called whenever the color set is changed.
    /// </summary>
    /// <remarks>
    /// This function will test for if the connection is established
    /// </remarks>
    private void SendColorSet()
    {
      lock (this)
      {
        if (m_conn != null)
        {
          // note: can't use linq here since ObjectCollection doesn't implement
          // IEnumerable<T>
          ColorSetEntry[] entries =
            new ColorSetEntry[coloring_color_set.Items.Count];
          for (int i = 0; i < entries.Length; ++i)
          {
            entries[i] = (ColorSetEntry)coloring_color_set.Items[i];
          }
          
          // create and send the message
          MsgColorSet msg = new MsgColorSet(entries);
          msg.Send(m_conn.GetStream());
        }
      }
    }

    /// <summary>
    /// Sends the current color sequence to the rendering server.  This should
    /// be called whenever the color sequence is changed.
    /// </summary>
    private void SendColorSeq()
    {
      lock (this)
      {
        if (m_conn != null)
        {
          // note: can't use linq here since ObjectCollection doesn't implement
          // IEnumerable<T>
          ColorSeqEntry[] entries =
            new ColorSeqEntry[coloring_color_seq.Items.Count];
          for (int i = 0; i < entries.Length; ++i)
          {
            entries[i] = (ColorSeqEntry)coloring_color_seq.Items[i];
          }
          
          // create and send the message
          MsgColorSeq msg = new MsgColorSeq(entries);
          msg.Send(m_conn.GetStream());
        }
      }
    }
    
    /// <summary>
    /// Informs the server, if connected, of all the parameter values
    /// </summary>
    private void SyncServer()
    {
      lock (this)
      {
        if (m_conn != null)
        {
          // send all the current config data to the rendering server
          foreach (KeyValuePair<object,ParameterIDs> entry in m_ids)
          {
            NetworkCommon.Message msg = null;
            if (entry.Key is TextBox)
            {
              TextBox input = (TextBox)entry.Key;
              msg = new NetworkCommon.MsgSetString(entry.Value,input.Text);
            }
            else if (entry.Key is NumericUpDown)
            {
              NumericUpDown input = (NumericUpDown)entry.Key;
              msg = new NetworkCommon.MsgSetSingle(entry.Value,
                (float)input.Value);
            }
            else if (entry.Key is ComboBox)
            {
              ComboBox input = (ComboBox)entry.Key;
              msg = new NetworkCommon.MsgSetSingle(entry.Value,
                (float)input.SelectedIndex);
            }
            // ignore other types of UI elements
          
            // send the message
            if (msg != null)
            {
              msg.Send(m_conn.GetStream());
            }
          }
          SendColorSet();
          SendColorSeq();
        }
      }
    }

    /// <summary>
    /// Handles the user trying to connect to the rendering server
    /// </summary>
    private void connectToolStripMenuItem_Click(object sender,EventArgs e)
    {
      lock (this)
      {
        try
        {
          m_conn =
            new TcpClient("localhost",NetworkCommon.Constants.DefaultPort);
          
          // if we got here, then there is a connection
          connectToolStripMenuItem.Enabled = false;
          disconnectToolStripMenuItem.Enabled = true;
          
          // create the token for controlling the thread
          m_token = new CancellationTokenSource();
          
          // start the message receive thread (note: done after the updating of
          // the UI to avoid a race condition where the server sends back a
          // connection rejected message and the UI update happening after the
          // processing of that message)
          new Thread(() => ReceiveThread(m_token.Token,this)).Start();
        }
        catch (Exception err)
        {
          MessageBox.Show("Failed to connect to the server: " + err.Message);
        }
      }
      
      SyncServer();
    }

    /// <summary>
    /// Handles the user disconnecting from the server
    /// </summary>
    private void disconnectToolStripMenuItem_Click(object sender,EventArgs e)
    {
      lock (this)
      {
        if (m_conn != null)
        {
          // inform the server of the disconnect
          MsgCloseConnection msg = new MsgCloseConnection();
          msg.Send(m_conn.GetStream());
          
          // stop the receive thread
          m_token.Cancel();
          
          // clean up the connection
          ((IDisposable)m_conn).Dispose();
          m_conn = null;
          m_token = null;
        }
        
        // no more connection at this point
        connectToolStripMenuItem.Enabled = true;
        disconnectToolStripMenuItem.Enabled = false;
      }
    }

    /// <summary>
    /// Handles the form being closed
    /// </summary>
    private void FormMain_FormClosed(object sender,FormClosedEventArgs e)
    {
      // ensure the connection to the server and the associated thread are
      // terminated
      lock (this)
      {
        if (m_conn != null)
        {
          // inform the server of the disconnect
          MsgCloseConnection msg = new MsgCloseConnection();
          msg.Send(m_conn.GetStream());
          
          // stop the receive thread
          m_token.Cancel();
          
          // clean up the connection
          ((IDisposable)m_conn).Dispose();
          m_conn = null;
          m_token = null;
        }
      }
    }
    
    /// <summary>
    /// Handles the user creating a new file
    /// </summary>
    private void newToolStripMenuItem_Click(object sender,EventArgs e)
    {
      // 0 out the fields
      foreach (KeyValuePair<object,ParameterIDs> entry in m_ids)
      {
        if (entry.Key is TextBox)
        {
          TextBox input = (TextBox)entry.Key;
          input.Text = "0";
        }
        else if (entry.Key is NumericUpDown)
        {
          NumericUpDown input = (NumericUpDown)entry.Key;
          input.Value = 0;
        }
        else if (entry.Key is ComboBox)
        {
          ComboBox input = (ComboBox)entry.Key;
          input.SelectedIndex = 0;
        }
        // ignore other types of UI elements
      }
      
      // set the fields that shouldn't be 0 to their appropriate value
      cam_orbit_range.Value    = 10;
      emitter_spawn_time.Value = 10;
      emitter_scale_x.Value    = 1;
      emitter_scale_y.Value    = 1;
      emitter_scale_z.Value    = 1;
      pc_size.Value            = 0.01m;
      
      // empty the color data
      coloring_texture.Text = "";
      coloring_color_set.Items.Clear();
      coloring_color_seq.Items.Clear();
      coloring_color_seq_interpolate.Checked = false;
      
      // clear the save file fields
      m_filename = "";
      
      // inform the server of the updated values
      SyncServer();
    }

    /// <summary>
    /// Handles the user trying to open an saved file
    /// </summary>
    private void openToolStripMenuItem_Click(object sender,EventArgs e)
    {
      using (OpenFileDialog dlg = new OpenFileDialog())
      {
        dlg.Filter = "xml files (*.xml)|*.xml|All files (*.*)|*.*";
        dlg.FilterIndex = 1;
        dlg.RestoreDirectory = true;
        
        if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
        {
          m_filename = dlg.FileName;
          
          XmlDocument doc = new XmlDocument();
          try
          {
            doc.Load(m_filename);
            XmlElement category = doc.DocumentElement["camera"];
            XmlElement field    = category["look_x"];
            cam_lookat_x.Value = Convert.ToDecimal(field.InnerText);
            field  = category["look_y"];
            cam_lookat_y.Value = Convert.ToDecimal(field.InnerText);
            field  = category["look_z"];
            cam_lookat_z.Value = Convert.ToDecimal(field.InnerText);
            field  = category["range"];
            cam_orbit_range.Value = Convert.ToDecimal(field.InnerText);
            field  = category["yaw"];
            cam_orbit_yaw.Value = Convert.ToDecimal(field.InnerText);
            field  = category["pitch"];
            cam_orbit_pitch.Value = Convert.ToDecimal(field.InnerText);
            
            category = doc.DocumentElement["emitter"];
            field    = category["shape"];
            emitter_shape.Text = field.InnerText;
            field = category["time"];
            emitter_spawn_time.Value = Convert.ToDecimal(field.InnerText);
            field = category["rate"];
            emitter_spawn_rate.Value = Convert.ToDecimal(field.InnerText);
            field = category["pos_x"];
            emitter_pos_x.Value = Convert.ToDecimal(field.InnerText);
            field = category["pos_y"];
            emitter_pos_y.Value = Convert.ToDecimal(field.InnerText);
            field = category["pos_z"];
            emitter_pos_z.Value = Convert.ToDecimal(field.InnerText);
            field = category["scale_x"];
            emitter_scale_x.Value = Convert.ToDecimal(field.InnerText);
            field = category["scale_y"];
            emitter_scale_y.Value = Convert.ToDecimal(field.InnerText);
            field = category["scale_z"];
            emitter_scale_z.Value = Convert.ToDecimal(field.InnerText);
            field = category["yaw"];
            emitter_rot_yaw.Value = Convert.ToDecimal(field.InnerText);
            field = category["pitch"];
            emitter_rot_pitch.Value = Convert.ToDecimal(field.InnerText);
            field = category["roll"];
            emitter_rot_roll.Value = Convert.ToDecimal(field.InnerText);
            
            category = doc.DocumentElement["particle_control"];
            field    = category["shape"];
            pc_shape.Text = field.InnerText;
            field = category["lifetime"];
            pc_lifetime.Value = Convert.ToDecimal(field.InnerText);
            field = category["min_speed"];
            pc_min_speed.Value = Convert.ToDecimal(field.InnerText);
            field = category["max_speed"];
            pc_max_speed.Value = Convert.ToDecimal(field.InnerText);
            field = category["size"];
            pc_size.Value = Convert.ToDecimal(field.InnerText);
            field = category["acc_x"];
            pc_acc_x.Value = Convert.ToDecimal(field.InnerText);
            field = category["acc_y"];
            pc_acc_y.Value = Convert.ToDecimal(field.InnerText);
            field = category["acc_z"];
            pc_acc_z.Value = Convert.ToDecimal(field.InnerText);
            field = category["dispersion_type"];
            pc_dis_type.Text = field.InnerText;
            field = category["dir_x"];
            pc_dis_dir_x.Value = Convert.ToDecimal(field.InnerText);
            field = category["dir_y"];
            pc_dis_dir_y.Value = Convert.ToDecimal(field.InnerText);
            field = category["dir_z"];
            pc_dis_dir_z.Value = Convert.ToDecimal(field.InnerText);
            field = category["min_x"];
            pc_dis_angle_min_x.Value = Convert.ToDecimal(field.InnerText);
            field = category["max_x"];
            pc_dis_angle_max_x.Value = Convert.ToDecimal(field.InnerText);
            field = category["min_y"];
            pc_dis_angle_min_y.Value = Convert.ToDecimal(field.InnerText);
            field = category["max_y"];
            pc_dis_angle_max_y.Value = Convert.ToDecimal(field.InnerText);
            
            category = doc.DocumentElement["particle_coloring"];
            coloring_color_set.BeginUpdate();
            coloring_color_seq.BeginUpdate();
            coloring_color_set.Items.Clear();
            coloring_color_seq.Items.Clear();
            field = category["mode"];
            coloring_mode.Text = field.InnerText;
            field = category["texture"];
            coloring_texture.Text = field.InnerText;
            field = category["color_set"];
            foreach (XmlElement entry in field.ChildNodes)
            {
              if (entry.Name == "entry")
              {
                Vector3 color;
                float   prob = 0;
                
                color.X = Convert.ToSingle(entry["red"].InnerText);
                color.Y = Convert.ToSingle(entry["green"].InnerText);
                color.Z = Convert.ToSingle(entry["blue"].InnerText);
                prob    = Convert.ToSingle(entry["chance"].InnerText);
                
                coloring_color_set.Items.Add(new ColorSetEntry(color,prob));
              }
            }
            field = category["color_seq"];
            foreach (XmlElement entry in field.ChildNodes)
            {
              if (entry.Name == "entry")
              {
                Vector3 color;
                float   life = 0;
                
                color.X = Convert.ToSingle(entry["red"].InnerText);
                color.Y = Convert.ToSingle(entry["green"].InnerText);
                color.Z = Convert.ToSingle(entry["blue"].InnerText);
                life    = Convert.ToSingle(entry["percent"].InnerText);
                
                coloring_color_seq.Items.Add(new ColorSeqEntry(color,life));
              }
              else if (entry.Name == "interpolate")
              {
                coloring_color_seq_interpolate.Checked =
                  entry.InnerText == "0";
              }
            }
            coloring_color_set.EndUpdate();
            coloring_color_seq.EndUpdate();
          }
          catch (Exception err)
          {
            MessageBox.Show("Failed to load \"" + m_filename + "\".  Error \""
              + err.Message + "\"","Error");
            m_filename = "";
          }
        }
      }
    }
    
    /// <summary>
    /// Handles the user trying save the current settings to a file
    /// </summary>
    private void saveToolStripMenuItem_Click(object sender,EventArgs e)
    {
      if (m_filename == "")
      {
        // if this is the first save, use the save as functionality
        saveAsToolStripMenuItem_Click(sender,e);
      }
      else
      {
        XmlWriterSettings settings = new XmlWriterSettings();
        settings.Indent = true;
        using (XmlWriter writer = XmlWriter.Create(m_filename,settings))
        {
          writer.WriteStartElement("particle_system");
          
          writer.WriteStartElement("camera");
          writer.WriteStartElement("look_x");
          writer.WriteString(cam_lookat_x.Value.ToString());
          writer.WriteEndElement(); // look_x
          writer.WriteStartElement("look_y");
          writer.WriteString(cam_lookat_y.Value.ToString());
          writer.WriteEndElement(); // look_y
          writer.WriteStartElement("look_z");
          writer.WriteString(cam_lookat_z.Value.ToString());
          writer.WriteEndElement(); // look_z
          writer.WriteStartElement("range");
          writer.WriteString(cam_orbit_range.Value.ToString());
          writer.WriteEndElement(); // range
          writer.WriteStartElement("yaw");
          writer.WriteString(cam_orbit_yaw.Value.ToString());
          writer.WriteEndElement(); // yaw
          writer.WriteStartElement("pitch");
          writer.WriteString(cam_orbit_pitch.Value.ToString());
          writer.WriteEndElement(); // pitch
          writer.WriteEndElement(); // camera
          
          writer.WriteStartElement("emitter");
          writer.WriteStartElement("shape");
          writer.WriteString(emitter_shape.Text);
          writer.WriteEndElement(); // shape
          writer.WriteStartElement("time");
          writer.WriteString(emitter_spawn_time.Value.ToString());
          writer.WriteEndElement(); // time
          writer.WriteStartElement("rate");
          writer.WriteString(emitter_spawn_rate.Value.ToString());
          writer.WriteEndElement(); // rate
          writer.WriteStartElement("pos_x");
          writer.WriteString(emitter_pos_x.Value.ToString());
          writer.WriteEndElement(); // pos_x
          writer.WriteStartElement("pos_y");
          writer.WriteString(emitter_pos_y.Value.ToString());
          writer.WriteEndElement(); // pos_y
          writer.WriteStartElement("pos_z");
          writer.WriteString(emitter_pos_z.Value.ToString());
          writer.WriteEndElement(); // pos_z
          writer.WriteStartElement("scale_x");
          writer.WriteString(emitter_scale_x.Value.ToString());
          writer.WriteEndElement(); // scale_x
          writer.WriteStartElement("scale_y");
          writer.WriteString(emitter_scale_y.Value.ToString());
          writer.WriteEndElement(); // scale_y
          writer.WriteStartElement("scale_z");
          writer.WriteString(emitter_scale_z.Value.ToString());
          writer.WriteEndElement(); // scale_z
          writer.WriteStartElement("yaw");
          writer.WriteString(emitter_rot_yaw.Value.ToString());
          writer.WriteEndElement(); // yaw
          writer.WriteStartElement("pitch");
          writer.WriteString(emitter_rot_pitch.Value.ToString());
          writer.WriteEndElement(); // pitch
          writer.WriteStartElement("roll");
          writer.WriteString(emitter_rot_roll.Value.ToString());
          writer.WriteEndElement(); // roll
          writer.WriteEndElement(); // emitter
          
          writer.WriteStartElement("particle_control");
          writer.WriteStartElement("shape");
          writer.WriteString(pc_shape.Text);
          writer.WriteEndElement(); // shape
          writer.WriteStartElement("lifetime");
          writer.WriteString(pc_lifetime.Value.ToString());
          writer.WriteEndElement(); // lifetime
          writer.WriteStartElement("min_speed");
          writer.WriteString(pc_min_speed.Value.ToString());
          writer.WriteEndElement(); // min_speed
          writer.WriteStartElement("max_speed");
          writer.WriteString(pc_max_speed.Value.ToString());
          writer.WriteEndElement(); // max_speed
          writer.WriteStartElement("size");
          writer.WriteString(pc_size.Value.ToString());
          writer.WriteEndElement(); // size
          writer.WriteStartElement("acc_x");
          writer.WriteString(pc_acc_x.Value.ToString());
          writer.WriteEndElement(); // acc_x
          writer.WriteStartElement("acc_y");
          writer.WriteString(pc_acc_y.Value.ToString());
          writer.WriteEndElement(); // acc_y
          writer.WriteStartElement("acc_z");
          writer.WriteString(pc_acc_z.Value.ToString());
          writer.WriteEndElement(); // acc_z
          writer.WriteStartElement("dispersion_type");
          writer.WriteString(pc_dis_type.Text);
          writer.WriteEndElement(); // dispersion_type
          writer.WriteStartElement("dir_x");
          writer.WriteString(pc_dis_dir_x.Value.ToString());
          writer.WriteEndElement(); // dir_x
          writer.WriteStartElement("dir_y");
          writer.WriteString(pc_dis_dir_y.Value.ToString());
          writer.WriteEndElement(); // dir_y
          writer.WriteStartElement("dir_z");
          writer.WriteString(pc_dis_dir_z.Value.ToString());
          writer.WriteEndElement(); // dir_z
          writer.WriteStartElement("min_x");
          writer.WriteString(pc_dis_angle_min_x.Value.ToString());
          writer.WriteEndElement(); // min_x
          writer.WriteStartElement("max_x");
          writer.WriteString(pc_dis_angle_max_x.Value.ToString());
          writer.WriteEndElement(); // max_x
          writer.WriteStartElement("min_y");
          writer.WriteString(pc_dis_angle_min_y.Value.ToString());
          writer.WriteEndElement(); // min_y
          writer.WriteStartElement("max_y");
          writer.WriteString(pc_dis_angle_max_y.Value.ToString());
          writer.WriteEndElement(); // max_y
          writer.WriteEndElement(); // particle_control
          
          writer.WriteStartElement("particle_coloring");
          writer.WriteStartElement("mode");
          writer.WriteString(coloring_mode.Text);
          writer.WriteEndElement(); // mode
          writer.WriteStartElement("texture");
          writer.WriteString(coloring_texture.Text);
          writer.WriteEndElement(); // texture
          writer.WriteStartElement("color_set");
          foreach (object obj in coloring_color_set.Items)
          {
            ColorSetEntry entry = (ColorSetEntry)obj;
            writer.WriteStartElement("entry");
            writer.WriteStartElement("red");
            writer.WriteString(entry.Color.X.ToString());
            writer.WriteEndElement(); // red
            writer.WriteStartElement("green");
            writer.WriteString(entry.Color.Y.ToString());
            writer.WriteEndElement(); // green
            writer.WriteStartElement("blue");
            writer.WriteString(entry.Color.Z.ToString());
            writer.WriteEndElement(); // blue
            writer.WriteStartElement("chance");
            writer.WriteString(entry.Chance.ToString());
            writer.WriteEndElement(); // chance
            writer.WriteEndElement(); // entry
          }
          writer.WriteEndElement(); // color_set
          writer.WriteStartElement("color_seq");
          foreach (object obj in coloring_color_seq.Items)
          {
            ColorSeqEntry entry = (ColorSeqEntry)obj;
            writer.WriteStartElement("entry");
            writer.WriteStartElement("red");
            writer.WriteString(entry.Color.X.ToString());
            writer.WriteEndElement(); // red
            writer.WriteStartElement("green");
            writer.WriteString(entry.Color.Y.ToString());
            writer.WriteEndElement(); // green
            writer.WriteStartElement("blue");
            writer.WriteString(entry.Color.Z.ToString());
            writer.WriteEndElement(); // blue
            writer.WriteStartElement("percent");
            writer.WriteString(entry.Life.ToString());
            writer.WriteEndElement(); // percent
            writer.WriteEndElement(); // entry
          }
          writer.WriteStartElement("interpolate");
          writer.WriteString(
            coloring_color_seq_interpolate.Checked ? "1" : "0");
          writer.WriteEndElement(); // interpolate
          writer.WriteEndElement(); // color_seq
          writer.WriteEndElement(); // particle_coloring
          
          writer.WriteEndElement(); // particle_system
        }
      }
    }
    
    /// <summary>
    /// Handles the user trying save the current settings to a new file
    /// </summary>
    private void saveAsToolStripMenuItem_Click(object sender,EventArgs e)
    {
      using (SaveFileDialog dlg = new SaveFileDialog())
      {
        dlg.Filter = "xml files (*.xml)|*.xml|All files (*.*)|*.*";
        dlg.FilterIndex = 1;
        dlg.RestoreDirectory = true;
        
        if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
        {
          m_filename = dlg.FileName;
          
          // now that the filename is set, use the save menu itme to do the
          // actual writing
          saveToolStripMenuItem_Click(sender,e);
        }
      }
    }
    
    /// <summary>
    /// Takes care of processing messages from the server
    /// </summary>
    /// <param name="msg">
    /// received message
    /// </param>
    private void ReceiveMsg(NetworkCommon.Message msg)
    {
      bool close_conn = false;
      
      switch (msg.GetMessageID())
      {
        case MessageIDs.CLOSE_CONNECTION:
          MessageBox.Show("Server closed the connection");
          close_conn = true;
          break;
          
        case MessageIDs.CONNECTION_REJECTED:
          MessageBox.Show("Server rejected the connection");
          close_conn = true;
          break;
                
        default:
          MessageBox.Show("Received unexpected message type of "
            + msg.GetMessageID() + " from the server");
          break;
      }
      
      if (close_conn)
      {
        lock (this)
        {
          // stop the receive thread
          m_token.Cancel();
          
          // clean up the connection
          ((IDisposable)m_conn).Dispose();
          m_conn = null;
          m_token = null;
          
          // update the UI
          connectToolStripMenuItem.Enabled = true;
          disconnectToolStripMenuItem.Enabled = false;
        }
      }
    }
    
    /// <summary>
    /// Thread function for handling messages received from the server
    /// </summary>
    /// <param name="token">
    /// Token to use for determining when to end the thread
    /// </param>
    /// <param name="form">
    /// FormMain instance that is running this thread
    /// </param>
    static private void ReceiveThread(CancellationToken token,FormMain form)
    {
      while (!token.IsCancellationRequested)
      {
        // declare the msg variable here so that the lock is only held while
        // reading the message.  This is needed so that Invoke doesn't get into
        // lock contention with the lock for reading from the socket
        NetworkCommon.Message msg = null;
        
        // check if there is data waiting
        lock (form)
        {
          if (form.m_conn != null && form.m_conn.Available > 0)
          {
            msg = NetworkCommon.Message.Receive(form.m_conn.GetStream());
          }
        }
        
        // process any received message
        if (msg != null)
        {
          form.Invoke(form.receive_msg,new object[] { msg } );
        }
        // if no message was received, sleep
        else
        {
          Thread.Sleep(10);
        }
      }
    }
    
    /// <summary>
    /// Delegate type for processing messages from the server
    /// </summary>
    /// <param name="msg">
    /// Message that was received
    /// </param>
    private delegate void ReceivedMsgDelegate(NetworkCommon.Message msg);
    
    /// <summary>
    /// maps UI elements to editable field ids.  This is used so that all
    /// single value UI elements can use one listener for value changed, and
    /// avoid a long if/elseif block
    /// </summary>
    private Dictionary<object,ParameterIDs> m_ids;
    
    /// <summary>
    /// connection to the renderer
    /// </summary>
    /// <remarks>
    /// Accessing this should only be done while holding a lock on the
    /// FormMain instance
    /// </remarks>
    private TcpClient m_conn;
    
    /// <summary>
    /// token for informing the receive thread of when to stop
    /// </summary>
    private CancellationTokenSource m_token;
    
    /// <summary>
    /// delegate for Invoke to call so that the processing of network messages
    /// can interface properly with the UI
    /// </summary>
    private ReceivedMsgDelegate receive_msg;
    
    /// <summary>
    /// path to the saved file
    /// </summary>
    private string m_filename;
  }
}
