﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace XNA_Renderer.ParticleSystem.DispersionControl
{
  /// <summary>
  /// Sets particle movement to be a particular direction in emitter local
  /// space
  /// </summary>
  class Directional : DispersionBase
  {
    /// <summary>
    /// Creates a directional dispersion instance
    /// </summary>
    public Directional()
    {
      m_dir = Vector4.Zero;
    }
    
    /// <summary>
    /// Sets the x component of the direction to send particles in
    /// </summary>
    /// <param name="x">
    /// x component, in emitter local space, to send particles in
    /// </param>
    public void SetDirX(float x)
    {
      m_dir.X = x;
    }
    /// <summary>
    /// Sets the y component of the direction to send particles in
    /// </summary>
    /// <param name="y">
    /// y component, in emitter local space, to send particles in
    /// </param>
    public void SetDirY(float y)
    {
      m_dir.Y = y;
    }
    /// <summary>
    /// Sets the z component of the direction to send particles in
    /// </summary>
    /// <param name="z">
    /// z component, in emitter local space, to send particles in
    /// </param>
    public void SetDirZ(float z)
    {
      m_dir.Z = z;
    }
    
    /// <summary>
    /// Determines the direction of a newly spawned particle
    /// </summary>
    /// <returns>
    /// unit vector representing the direction of the particle in emitter local
    /// space
    /// </returns>
    public override Vector4 GetDirection()
    {
      return Vector4.Normalize(m_dir);
    }
    
    /// <summary>
    /// direction to send particles in, in emitter local space
    /// </summary>
    private Vector4 m_dir;
  }
}
