﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace XNA_Renderer.ParticleSystem.DispersionControl
{
  /// <summary>
  /// Sets particle movement to be in a partial sphere around the particle's
  /// spawned location
  /// </summary>
  class PartialSphere : DispersionBase
  {
    /// <summary>
    /// Determines the direction of a newly spawned particle
    /// </summary>
    /// <returns>
    /// unit vector representing the direction of the particle in emitter local
    /// space
    /// </returns>
    public override Vector4 GetDirection()
    {
      float  yaw   = StaticRandom.Rng.NextFloat() * (MaxX - MinX) + MinX;
      float  pitch = StaticRandom.Rng.NextFloat() * (MaxY - MinY) + MinY;
      
      Vector4 dir = Vector4.UnitZ;
      Matrix  rot = Matrix.CreateFromYawPitchRoll(yaw,pitch,0);
      
      return Vector4.Transform(dir,rot);
    }
    
    /// <summary>
    /// min angle around the x-axis, in radians and in the emitter's local
    /// space, to use for the direction of particles
    /// </summary>
    public float MinX { get; set; }
    
    /// <summary>
    /// max angle around the x-axis, in radians and in the emitter's local
    /// space, to use for the direction of particles
    /// </summary>
    /// <remarks>
    /// Having this be a range would be more efficient when determing particle
    /// direction, but makes it more complicated for setting min/max for this
    /// rotation
    /// </remarks>
    public float MaxX { get; set; }
    
    /// <summary>
    /// min angle around the y-axis, in radians and in the emitter's local
    /// space, to use for the direction of particles
    /// </summary>
    public float MinY { get; set; }
    
    /// <summary>
    /// maxn angle around the y-axis, in radians and in the emitter's local
    /// space, to use for the direction of particles
    /// </summary>
    /// <remarks>
    /// Having this be a range would be more efficient when determing particle
    /// direction, but makes it more complicated for setting min/max for this
    /// rotation
    /// </remarks>
    public float MaxY { get; set; }
  }
}
