﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace XNA_Renderer.ParticleSystem.EmitterShapes
{
  /// <summary>
  /// Hemisphere emitter that creates particles in a unit sphere and z >= 0 in
  /// the emitter's local space, centered at the origin
  /// </summary>
  class Hemisphere : EmitterShapeBase
  {
    /// <summary>
    /// Retrieves a position for a new particle based on the particular
    /// subtype's intented shape
    /// </summary>
    /// <returns>
    /// position for a new particle in the emitter's local space
    /// </returns>
    /// <remarks>
    /// This does not take into account scaling, rotation, etc since the
    /// particle system itself does that
    /// </remarks>
    public override Vector4 SpawnPosition()
    {
      float x = StaticRandom.Rng.NextFloat() * 2 - 1;
      float y = StaticRandom.Rng.NextFloat() * 2 - 1;
      float z = (float)Math.Sqrt(1 - x * x - y * y);
      
      return new Vector4(x,y,z,1);
    }
  }
}
