﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace XNA_Renderer.ParticleSystem.EmitterShapes
{
  /// <summary>
  /// Square perimeter emitter that creates particles along the edge of a 1x1
  /// region in the xz plane in the emitter's local space, centered at the
  /// origin
  /// </summary>
  class SquarePerimeter : EmitterShapeBase
  {
    /// <summary>
    /// Retrieves a position for a new particle based on the particular
    /// subtype's intented shape
    /// </summary>
    /// <returns>
    /// position for a new particle in the emitter's local space
    /// </returns>
    /// <remarks>
    /// This does not take into account scaling, rotation, etc since the
    /// particle system itself does that
    /// </remarks>
    public override Vector4 SpawnPosition()
    {
      float val = StaticRandom.Rng.NextFloat();
      
      if (val < .25f)
      {
        return new Vector4(val * 4 - .5f,0,-.5f,1);
      }
      else if (val < .5f)
      {
        return new Vector4((val - .25f) * 4 - .5f,0,.5f,1);
      }
      else if (val < .75f)
      {
        return new Vector4(-.5f,0,(val - .5f) * 4 - .5f,1);
      }
      else
      {
        return new Vector4(.5f,0,(val - .75f) * 4 - .5f,1);
      }
    }
  }
}
