﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace XNA_Renderer.ParticleSystem
{
  /// <summary>
  /// Stores settings for the particles
  /// </summary>
  class ParticleSettings
  {
    /// <summary>
    /// Creates a particle settings instance
    /// </summary>
    public ParticleSettings()
    {
      m_acc = Vector4.Zero;
    }
    
    /// <summary>
    /// Sets the x component of the particle acceleration
    /// </summary>
    /// <param name="y">
    /// x component of the particle acceleration in units/second^2 in the
    /// emitter's local space
    /// </param>
    public void SetAccX(float x)
    {
      m_acc.X = x;
    }
    
    /// <summary>
    /// Sets the y component of the particle acceleration
    /// </summary>
    /// <param name="x">
    /// y component of the particle acceleration in units/second^2 in the
    /// emitter's local space
    /// </param>
    public void SetAccY(float y)
    {
      m_acc.Y = y;
    }
    
    /// <summary>
    /// Sets the z component of the particle acceleration
    /// </summary>
    /// <param name="z">
    /// z component of the particle acceleration in units/second^2 in the
    /// emitter's local space
    /// </param>
    public void SetAccZ(float z)
    {
      m_acc.Z = z;
    }
    
    /// <summary>
    /// Retrieves the acceleration vector for particles
    /// </summary>
    /// <returns>
    /// acceleration vector in units/second^2 in the emitter's local space
    /// </returns>
    public Vector4 GetAcc()
    {
      return m_acc;
    }
    
    /// <summary>
    /// Shape of the particles
    /// </summary>
    public NetworkCommon.ParticleShapes Shape { get; set; }
    
    /// <summary>
    /// How long in seconds particles should live for, in seconds
    /// </summary>
    public float Lifetime { get; set; }
    
    /// <summary>
    /// Size of the particles in units
    /// </summary>
    public float Size { get; set; }
    
    /// <summary>
    /// Minimium speed in units/second that particles should move at
    /// </summary>
    public float MinSpeed { get; set; }
    
    /// <summary>
    /// Maximium speed in units/second that particles should move at
    /// </summary>
    public float MaxSpeed { get; set; }
    
    /// <summary>
    /// Acceleration to apply to the speed of particles in units/second^2 in
    /// the emitter's local space
    /// </summary>
    private Vector4 m_acc;
  }
}
