﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace XNA_Renderer
{
  /// <summary>
  /// Provides a single pseudo-random number generator for the game by holding
  /// a single static System.Random instance
  /// </summary>
  static public class StaticRandom
  {
    /// <summary>
    /// Static constructor for the type.  It takes care of settting up the
    /// pseudo-random number generator
    /// </summary>
    static StaticRandom()
    {
      Rng = new Random();
    }
    
    /// <summary>
    /// pseudo-random number generator
    /// </summary>
    static public Random Rng { get; private set; }
    
    /// <summary>
    /// Extension method for Random to take care of casting Random's double
    /// return type to a float
    /// </summary>
    /// <param name="rng">
    /// pseudo random number generator
    /// </param>
    /// <returns>
    /// floating point value in the range [0,1]
    /// </returns>
    static public float NextFloat(this Random rng)
    {
      return (float)rng.NextDouble();
    }
  }
}
